package ms.itsecteam.warpdrink.manager;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.util.Log;
import android.widget.Toast;
import java.util.List;
import ms.itsecteam.warpdrink.MainActivity;
import ms.itsecteam.warpdrink.R;
import ms.itsecteam.warpdrink.data.SQLiteService;
import ms.itsecteam.warpdrink.data.models.local.Category;
import ms.itsecteam.warpdrink.data.models.local.User;
import ms.itsecteam.warpdrink.data.RestService;
import ms.itsecteam.warpdrink.data.models.rest.RestCategory;
import ms.itsecteam.warpdrink.data.models.rest.RestProduct;
import ms.itsecteam.warpdrink.data.models.rest.RestTransaction;
import ms.itsecteam.warpdrink.data.models.rest.RestUser;
import ms.itsecteam.warpdrink.orderList.Order;
import retrofit.Callback;
import retrofit.GsonConverterFactory;
import retrofit.Response;
import retrofit.Retrofit;

public class MainManager {

    private RestService restService;
    private MainActivity mainActivity;
    private SQLiteService sqLiteService;
    private UiManager uiManager;
    private DataManager dataManager;

    private static MainManager instance = new MainManager();

    public static MainManager getInstance() {
        return instance;
    }

    private MainManager() {
        this.sqLiteService = SQLiteService.getInstance();
        this.uiManager = UiManager.getInstance();
        this.dataManager = DataManager.getInstance();
        this.restService = this.createRestService();
    }

    private RestService createRestService() {
        Retrofit retrofit = new Retrofit.Builder()
                //.baseUrl("http://10.0.1.134:8000/api/")
                //.baseUrl("https://infra-test.warpzone.ms/api/")
                .baseUrl("http://192.168.0.194:8000/api/")
                .addConverterFactory(GsonConverterFactory.create())
                .build();
        return retrofit.create(RestService.class);
    }

    public void init(MainActivity mainActivity) {
        this.mainActivity = mainActivity;
        this.uiManager.initUi(this.mainActivity);

        this.refreshData();
    }

    public void refreshData() {
        Log.d("REST", "Refreshing user data");

        this.restService.getAllUser().enqueue(new Callback<List<RestUser>>() {
            @Override
            public void onResponse(Response<List<RestUser>> response, Retrofit retrofit) {
                List<User> userList = MainManager.getInstance().sqLiteService.refreshUserData(response.body());
                if (userList != null) {
                    MainManager.getInstance().uiManager.refreshUserData(userList);
                }
                Toast.makeText(MainManager.getInstance().mainActivity, "Refresh successful", Toast.LENGTH_LONG).show();
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
            }
        });

        this.restService.getAllCategories().enqueue(new Callback<List<RestCategory>>() {
            @Override
            public void onResponse(Response<List<RestCategory>> response, Retrofit retrofit) {
                List<Category> categoryList = MainManager.getInstance().sqLiteService.refreshCategoryData(response.body());
                if (categoryList != null) {
                    MainManager.getInstance().uiManager.showCategoryButtons();
                }
                Toast.makeText(MainManager.getInstance().mainActivity, "Refresh successful", Toast.LENGTH_LONG).show();
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
            }
        });

        this.restService.getAllProducts().enqueue(new Callback<List<RestProduct>>() {
            @Override
            public void onResponse(Response<List<RestProduct>> response, Retrofit retrofit) {
                List<ms.itsecteam.warpdrink.data.models.local.Product> productList = MainManager.getInstance().sqLiteService.refreshProductData(response.body());
                if (productList != null) {
                    MainManager.getInstance().uiManager.refreshProductData(productList);
                }
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
            }
        });

    }
/**
    public boolean addUser(String name) {
        Log.d("USER", "Start");
         if(User.getByName(name) == null) {
             User user = new User();
             user.setUserid(name);
             Log.d("USER", "Weter");
             this.restService.addUser(RestUser.fromLocalUser(user)).enqueue(new Callback<RestUser>() {
                 @Override
                 public void onResponse(Response<RestUser> response, Retrofit retrofit) {
                     MainManager.getInstance().refreshData();
                 }

                 @Override
                 public void onFailure(Throwable t) {
                     Log.d("Error", t.getMessage());
                 }
             });

             return true;
         }
        return false;
    }
**/
    public void setCurrentUser(User user) {
        if (user != null) {
            this.dataManager.setCurrentUser(user);
            this.uiManager.setCurrentUser(user);
        }
    }

    public User getCurrentUser() {
        return this.dataManager.getCurrentUser();
    }
    public void clearCurrentUser() {
        this.dataManager.clearCurrentUser();
        this.uiManager.clearCurrentUser();
    }

    public void addOrder(Order order) {
        DataManager.getInstance().addOrder(order);
        this.uiManager.refreshTotalTextView(DataManager.getInstance().getTotalAmount());
    }

    public void removeOrder(Order order) {
        DataManager.getInstance().removeOrder(order);
        this.uiManager.removeOrder(order);
        this.uiManager.refreshTotalTextView(DataManager.getInstance().getTotalAmount());

    }
    public void chargeAmount(double amount) {
        User currentUser = this.dataManager.getCurrentUser();
        RestTransaction t = new RestTransaction();
        t.setAmount(amount);
        t.setTrans_type(1);
        t.setCash_paid(true);
        MainManager.getInstance().getRestService().saveTransaction(currentUser.getUserid(), t).enqueue(new Callback<Void>() {
            @Override
            public void onResponse(Response<Void> response, Retrofit retrofit) {
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
            }
        });
        MainManager.getInstance().getRestService().getSingleUser(currentUser.getUserid()).enqueue(new Callback<RestUser>() {
            @Override
            public void onResponse(Response<RestUser> response, Retrofit retrofit) {
                DataManager.getInstance().getCurrentUser().setCredit(response.body().getCredit());
                MainManager.getInstance().uiManager.refreshCreditTextView(DataManager.getInstance().getCurrentUser().getCredit());
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
            }
        });


    }
    public void performPayment(Boolean cash) {
        User user = DataManager.getInstance().getCurrentUser();
        double totalAmount = DataManager.getInstance().getTotalAmount();

        if (totalAmount > 0.0 && user != null){
            if (DataManager.getInstance().getCurrentUser().getCredit()>=totalAmount || cash){
                DataManager.getInstance().performPayment(true);
                MainManager.getInstance().clearCurrentUser();
            } else {
                Toast.makeText(mainActivity, mainActivity.getResources().getString(R.string.please_charge), Toast.LENGTH_LONG).show();
            }
        }
    }


    public MainActivity getMainActivity() {
        return mainActivity;
    }

    public void setMainActivity(MainActivity mainActivity) {
        this.mainActivity = mainActivity;

    }

    public RestService getRestService() { return restService; }


    public void saveCardId() {
        final String card_id = this.dataManager.getLastCardId();

        if (card_id != null) {
            new AlertDialog.Builder(this.mainActivity)
                .setTitle(this.mainActivity.getResources().getString(R.string.confirm_learn_card))
                .setMessage("Die karten id "+card_id+" wird dem Nutzer "+this.getCurrentUser()+" hinzugefügt")
                .setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        MainManager.getInstance().getCurrentUser().setCardId(card_id);
                        MainManager.getInstance().dataManager.saveCurrentUser();

                    }
                })
                .setNegativeButton(android.R.string.no, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                    }
                })
                .setIcon(android.R.drawable.ic_dialog_alert)
                .show();
        }
    }
}
